///////////////////////////////////////////////////////////
//     TMeter Delphi Visual Component Library
//     by I MD. CIPTAYASA(IMDC)
//     ******************************
//     Description : Speedometer-like appearance Delphi's VCL
//     Version     : 1.0.0
//     Base Class  : TCustomControl
//     Programmer  : IMDC
//     Date        : Jan'2001
//     Compiler    : Delphi 3 (it'd be straightforward task porting to Delphi 5)
//
//     ****************************************************
//     Feel free to modify to fit your needs
//////////////////////////////////////////////////////////

unit Meter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf ;

type
  TMeter = class(TCustomControl)
  private
    FFrameColor     : TColor;
    FBackColor      : TColor;
    FFont           : TFont;
    FCaption        : TCaption;
    FMinValue       : integer;
    FMaxValue       : integer;
    FTicks          : cardinal;
    FPosition       : integer;
    FFramed         : Boolean;
    FNeedleColor    : TColor;
    FAngle          : integer;

    procedure DrawFace(ABitmap : TBitmap);
    procedure PlaceLabel(ACanvas : TCanvas;X,Y : integer; S : string);
    procedure GetDrawingArea(var Buffer : TRect);
    procedure SetNeedleColor(Value : TColor);
    procedure DrawMarkPosition(ACanvas:TCanvas;Position:integer;Erase:Boolean);
    procedure DrawTicks(ACanvas : TCanvas);
    procedure DrawCaption(ACanvas : TCanvas);

    procedure SetTicks(Value : Cardinal);
    procedure SetFramed(Value : Boolean);
    procedure SetFrameColor(Value : TColor);
    procedure SetBackColor(Value : TColor);
    procedure SetCaption(Value : TCaption);
    procedure SetMinValue(Value : integer);
    procedure SetMaxVAlue(Value : integer);
    procedure SetPosition(Value : integer);
  protected
    procedure Click;override;
    procedure Paint;override;
    procedure WMSize(var Message:TMessage);message WM_SIZE;
  public
    constructor Create(AOwner : TComponent);override;
  published
    property NeedleColor:TColor read FNeedleColor write SetNeedleColor default clRed;
    property FrameColor:TColor read FFrameColor write SetFrameColor default clAqua;
    property BackGroundColor:TColor read FBackColor write SetBackColor ;
    property MinValue:integer read FMinValue write SetMinValue;
    property Caption:TCaption read FCaption write SetCaption;
    property MaxValue:integer read FMaxValue write SetMaxValue;
    property Position:integer read FPosition write SetPosition;
    property Framed:Boolean read FFramed write SetFramed;
    property TicksFrequency:Cardinal read FTicks write SetTicks default 5;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
  end;

  TChipEditor = class(TComponentEditor)
    procedure Edit;override;
  end;

procedure Register;

implementation


procedure TChipEditor.Edit;
begin
  ShowMessage('TMeter - Delphi VCL'+#13#10+'Free Delphi VCL by IMDC ');
end;

constructor TMeter.Create(AOwner : TComponent);
begin
   inherited Create(AOwner);
   FBackColor  := $0080FFFF;
   FFrameColor := clMaroon;
   FNeedleColor:= clRed;
   FTicks      := 5;
   Framed      := true;
   FMinValue   := 0;
   FMaxValue   := 100;
   Width       := 115;
   Height      := 115;
   FPosition   := 0;
   TabStop     := false;
   FFont       := Font;
   FCaption    := Name;
end;

procedure TMeter.GetDrawingArea(var Buffer : TRect);
var
 R : TRect;
 W,H : integer;
begin
 R := ClientRect;
 W := R.Right - R.Left;
 H := R.Bottom - R.Top;

 // Make Width = Height

 if W > H then
    R.Right := H
 else
    R.Bottom := W;
 CopyRect(Buffer,R);
end;

// for use with DrawTicks method

procedure TMeter.PlaceLabel(ACanvas : TCanvas;X,Y : integer; S : string);
var
 W,H : integer;
begin
 W := ACanvas.TextWidth(S);
 H := ACanvas.TextHeight(S);
 with ACanvas do
  begin
    Font := FFont;
    Brush.Style := bsSolid;
    Brush.Color := FBackColor;
    Pen.Color   := FBackColor;
    TextOut(X - (W div 2),Y-(H div 2),S);
  end;
end;

procedure TMeter.DrawCaption(ACanvas : TCanvas);
var
 R     : TRect;
 Radius: integer;
 H,W: integer;
begin
  GetDrawingArea(R);
  W := R.Right - R.Left;
  H := R.Bottom - R.Top;
  Radius := H div 2;
  with ACanvas do
   begin
    InflateRect(R,-(W div 3) ,-(Radius div 4));
    R.Top := Radius + (Radius div 2)+3;
    Pen.Color := FBackColor;
    Brush.Style := bsSolid;
    Brush.Color:= FBackColor;
    Rectangle(R.Left,R.Top-5,R.Right,R.Bottom+5);
    DrawText(Handle,PChar(FCaption),Length(FCaption),R,
             DT_VCENTER or DT_CENTER or DT_SINGLELINE);
   end;
end;

// Draw values on frame
procedure TMeter.DrawTicks(ACanvas : TCanvas);
var
 Radius           : integer;
 CX,CY,X,Y        : integer;
 R                : TRect;
 Teta             : real;
 Num              : integer;
begin
   GetDrawingArea(R);
   Radius := (R.Right-R.Left) div 2;
   Num := (FMaxValue + FMinValue) div 2;
   CX:=R.Left+Radius;
   CY:=R.Top+Radius;
   Teta := Pi/180;
   PlaceLabel(ACanvas,CX,R.Top + 30,IntToStr(Num));
   X:=Round((Radius - 20)*Sin(-315*Teta));
   Y:=Round((Radius - 20)*Cos(-315*Teta));
   PlaceLabel(ACanvas,CX+X-5, CY+Y-5,IntToStr(FMaxValue));
   X:=Round((Radius - 20)*Sin(315*Teta));
   Y:=Round((Radius - 20)*Cos(315*Teta));
   PlaceLabel(ACanvas,CX+X+5 ,CY+Y-5,IntToStr(FMinValue));
   DrawCaption(ACanvas);
end;

// perform normalization
function MapToAngle(sMin,sMax,dMin,dMax,Value : integer):integer;
var
 Temp : real;
begin
 Temp := dMin +((dMax - dMin)/(sMax - sMin))*(Value-sMin);
 Result := Round(Temp);
end;

procedure TMeter.DrawMarkPosition(ACanvas:TCanvas;Position:integer;Erase:Boolean);
var
 R,RInner          : TRect;
 Diff,CX,CY,Radius : integer;
 Xi,Yi,Xo,Yo       : integer;
 Teta              : real;
 W                 : integer;
begin
  GetDrawingArea(R);
  GetDrawingArea(RInner);
  W := RInner.Right  div 30;
  Radius := (R.Right-R.Left) div 2;
  Diff := Radius div 6;
  CX:= (R.Right - R.Left) div 2;
  CY:= (R.Bottom - R.Top) div 2;
  FAngle := MapToAngle(FMinValue,FMaxValue,-135,135,Position);
  Teta := Pi/180;
  Xi:=Round((Radius - 5*Diff)*Sin(FAngle*Teta));
  Yi:=Round((Radius -5*Diff)*Cos(FAngle*Teta));
  Xo:=Round((Radius-2*Diff)*Sin(FAngle*Teta));
  Yo:=Round((Radius-2*Diff)*Cos(FAngle*Teta));

  with ACanvas do
   begin
    if Erase then
     Pen.Color := FBackColor
    else
     Pen.Color := FNeedleColor;
     Pen.Width := 2;
     MoveTo(CX-Xi,CY+Yi);
     LineTo(CX+Xo,CY-Yo);

     RInner := Rect(CX-W,CY-W,CX+W,CY+W);
    //Draw Inner Mark
     Brush.Color := FNeedleColor;
     Ellipse(RInner.Left,RInner.Top,RInner.Right,RInner.Bottom);
   end;
end;

//Draw the main appearance

procedure TMeter.DrawFace(ABitmap : TBitmap);
var
 i                     : integer;
 Radius                : integer;
 CX,CY,X,Y,X_1,Y_1     : integer;
 RFrame                : TRect;
 Teta                  : real;
begin
  GetDrawingArea(RFrame);
  with ABitmap.Canvas do
   begin
    if Framed then
     Pen.Color   := FFrameColor
    else
     Pen.Color   := FBackColor;
    Brush.Style := bsSolid;
    Brush.Color := FBackColor;
    Pen.Width   := 1;
    // Draw Outer Frame
    Ellipse(RFrame.Left,RFrame.Top,RFrame.Right,RFrame.Bottom);
    Pen.Color   := FFrameColor;
    InflateRect(RFrame,-5,-5);
    Radius := (RFrame.Right - RFrame.Left) div 2;
    CX:=RFrame.Left+Radius;
    CY:=RFrame.Top+Radius;
    Arc(RFrame.Left,RFrame.Top,RFrame.Right,RFrame.Bottom,
        RFrame.Right,RFrame.Bottom,RFrame.Left,RFrame.Bottom);
    Teta:= pi/180;
    for i:=-135 to 135 do
     begin
       X:=Round((Radius - 5)*Sin(i*Teta));
       Y:=Round((Radius - 5)*Cos(i*Teta));
       X_1:=Round(Radius*Sin(i*Teta));
       Y_1:=Round(Radius*Cos(i*Teta));
       if (i mod FTicks)=0 then
        begin
         MoveTo(CX+X,CY-Y);
         LineTo(CX+X_1,CY-Y_1);
        end;
     end;
     //MinValue mark
       X:=Round((Radius - 15)*Sin(-135*Teta));
       Y:=Round((Radius - 15)*Cos(-135*Teta));
       X_1:=Round(Radius*Sin(-135*Teta));
       Y_1:=Round(Radius*Cos(-135*Teta));
       MoveTo(CX+X,CY-Y);
       LineTo(CX+X_1,CY-Y_1);

     // MidValue mark
       X:=Round((Radius - 15)*Sin(0*Teta));
       Y:=Round((Radius - 15)*Cos(0*Teta));
       X_1:=Round(Radius*Sin(0*Teta));
       Y_1:=Round(Radius*Cos(0*Teta));
       MoveTo(CX+X,CY-Y);
       LineTo(CX+X_1,CY-Y_1);
     //maxValue mark
       X:=Round((Radius - 15)*Sin(135*Teta));
       Y:=Round((Radius - 15)*Cos(135*Teta));
       X_1:=Round(Radius*Sin(135*Teta));
       Y_1:=Round(Radius*Cos(135*Teta));
       MoveTo(CX+X,CY-Y);
       LineTo(CX+X_1,CY-Y_1);

   end;

   DrawTicks(ABitmap.Canvas);
   DrawMarkPosition(ABitmap.Canvas,FPosition,false);
end;

procedure TMeter.Paint;
var
 OffScreen:TBitmap;
begin
// Create OffScreen buffer to enhance performance
  try
   OffScreen:=TBitmap.Create;
   OffScreen.Width:=Width;
   OffScreen.Height:=Height;
   OffScreen.Canvas.Brush.Style := bsClear;
   OffScreen.Canvas.Handle:=Canvas.Handle;
   DrawFace(OffScreen);

   // Copy back to screen >><< BitBlt is faster than canvas Draw method
   BitBlt(Canvas.Handle,0,0,Width,Height,OffScreen.Canvas.Handle,0,0,SRCCOPY);
   finally
    OffScreen.Free;
  end;
  inherited Paint;
end;

//Enforce the smallest size that still maintain "Good-Look"

procedure TMeter.WMSize(var Message:TMessage);
begin
  if Width < 100 then
   Width := 100;
  if Height < 100 then
   Height := 100;
end;

procedure TMeter.SetFrameColor(Value : TColor);
begin
  if Value <> FFrameColor then
   begin
    FFrameColor := Value;
    Refresh;
   end;
end;

procedure TMeter.SetBackColor(Value : TColor);
begin
  if Value <> FBackColor then
   begin
    FBackColor := Value;
    Refresh;
   end;
end;

procedure TMeter.SetCaption(Value : TCaption);
begin
  if Value <> FCaption then
   begin
    FCaption := Value;
    DrawCaption(Canvas);
   end;
end;

procedure TMeter.SetMinValue(Value : integer);
begin
  if Value <> FMinValue then
   begin
    FMinValue := Value;
    Refresh;
   end;
end;

procedure TMeter.SetMaxVAlue(Value : integer);
begin
  if Value <> FMaxValue then
   begin
    FMaxValue := Value;
    Refresh;
   end;
end;

procedure TMeter.SetNeedleColor(Value : TColor);
begin
 if FNeedleColor <> Value then
   begin
    FNeedleColor := Value;
    Refresh;
   end;
end;

procedure TMeter.SetFramed(Value : Boolean);
begin
  if Value <> FFramed then
   begin
    FFramed := Value;
    Refresh;
   end;
end;

procedure TMeter.SetPosition(Value : integer);
begin
 if Value <> FPosition then
  if (Value >= FMinValue) and (Value <= FMaxValue) then
    begin
     DrawMarkPosition(Canvas,FPosition,true);
     FPosition := Value;
     DrawTicks(Canvas);
     DrawMarkPosition(Canvas,FPosition,false);
    end;
end;

procedure TMeter.SetTicks(Value : Cardinal);
begin
  if Value <> FTicks then
   begin
    FTicks := Value;
    Refresh;
   end;
end;

procedure TMeter.Click;
begin
  inherited;
end;

procedure Register;
begin
  RegisterComponents('Chip2000', [TMeter]);
  RegisterComponentEditor(TMeter,TChipEditor);
end;

end.
